﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using ThinkGeo.MapSuite.Core;

namespace GenerateTiling
{
    public partial class _Default : System.Web.UI.Page
    {
        // Set the tile width and height
        int tileWidth = 256;
        int tileHeight = 256;
        MapEngine mapEngine;
        Collection<ZoomLevel> zoomLevels;
        // Set the right map unit for your shape files
        GeographyUnit mapUnit = GeographyUnit.DecimalDegree;

        protected void Page_Load(object sender, EventArgs e)
        {
            Map1.MapUnit = GeographyUnit.DecimalDegree;
        }

        protected void btnGenerateTiling_Click(object sender, EventArgs e)
        {
            // Set the ZoomLevels you want to generate the cache images
            zoomLevels = GetAllZoomLevels();

            // Construct a new ShapeFileFeatureLayer, please speficy the file path for your own shape file, and here you can replace it with your WorldMapKitRenderLayer
            ShapeFileFeatureLayer shapeFileFeatureLayer = new ShapeFileFeatureLayer(@"C:\Program Files (x86)\ThinkGeo\Map Suite Desktop Full Edition 4.0\Samples\SampleData\Data\Countries02.shp");
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            ShapeFileFeatureLayer majorCitiesLayer = new ShapeFileFeatureLayer(@"C:\Program Files (x86)\ThinkGeo\Map Suite Desktop Full Edition 4.0\Samples\SampleData\Data\MajorCities.shp");
            majorCitiesLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.Capital1;
            majorCitiesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Get the extent that you want to cache
            shapeFileFeatureLayer.Open();
            RectangleShape cacheExtent = shapeFileFeatureLayer.GetBoundingBox();
            shapeFileFeatureLayer.Close();

            // Initialize the MapEngine object
            mapEngine = new MapEngine();
            mapEngine.ShowLogo = false;

            // Add contries02 layer to it
            mapEngine.StaticLayers.Add(shapeFileFeatureLayer);

            // Add major cities layes to it
            mapEngine.StaticLayers.Add(majorCitiesLayer);

            Collection<Feature> features = null;
            ShapeFileFeatureLayer layer = null;
            for (int i = 0; i < zoomLevels.Count; i++)
            {

                MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(zoomLevels[i].Scale, tileWidth, tileHeight, mapUnit);
                BitmapTileCache tileCache = new FileBitmapTileCache(txtCachePath.Text, string.Empty, TileImageFormat.Png, tileMatrix);
                RowColumnRange rowColumnRange = tileCache.TileMatrix.GetIntersectingRowColumnRange(cacheExtent);
                
                if (i > 10)
                {
                    layer = new ShapeFileFeatureLayer(@"c:\WorldMapKitData\USA\urban_dtl.shp", ShapeFileReadWriteMode.ReadOnly);
                    layer.Open();
                }

                for (long row = rowColumnRange.MinRowIndex; row <= rowColumnRange.MaxRowIndex; row++)
                {
                    for (long column = rowColumnRange.MinColumnIndex; column <= rowColumnRange.MaxColumnIndex; column++)
                    {
                        // Get tile extent for every tile image
                        RectangleShape tileExtent = tileCache.TileMatrix.GetCell(row, column).BoundingBox;

                        // If zoom level is larger than 10, and we just generate tiles within US urban areas
                        if (i > 10)
                        {
                            // We use GetFeaturesIntersecting method to judge whether current exent intersect with USA urban layer
                            features = layer.QueryTools.GetFeaturesIntersecting(tileExtent, ReturningColumnsType.NoColumns);
                            if (features.Count == 0)
                            {
                                continue;
                            }
                        }

                        SaveTile(tileCache, tileExtent);
                    }
                }
            }
        }

        private void SaveTile(BitmapTileCache tileCache, RectangleShape tileExtent)
        {
            // Generate cache for every tile image
            using (Bitmap bitmap = new Bitmap(tileWidth, tileHeight))
            {
                mapEngine.CurrentExtent = tileExtent;
                mapEngine.OpenAllLayers();
                mapEngine.DrawStaticLayers(bitmap, mapUnit);
                mapEngine.DrawDynamicLayers(bitmap, mapUnit);
                mapEngine.CloseAllLayers();

                BitmapTile tile = new BitmapTile(bitmap, tileExtent, tileCache.TileMatrix.Scale);
                tileCache.SaveTile(tile);
            }
        }

        private Collection<ZoomLevel> GetAllZoomLevels()
        {
            // We only add the top 5 ZoomLevels here for example. 
            // If you want to add more scales, you need to optimize the way generating tiles otherwise 
            // the bitmap we generated might be too big to be hold in memory.
            ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
            Collection<ZoomLevel> resultZoomLevels = new Collection<ZoomLevel>();
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel01);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel02);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel03);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel04);
            resultZoomLevels.Add(zoomLevelSet.ZoomLevel05);
            return resultZoomLevels;
        }

        protected void btnLoadMap_Click(object sender, EventArgs e)
        {
            Map1.StaticOverlay.ServerCache.CacheDirectory = txtCachePath.Text;
        }
    }
}
